aboutsummaryrefslogtreecommitdiff
path: root/app
diff options
context:
space:
mode:
authorGravatar Alexandre Alapetite <alexandre@alapetite.fr> 2015-05-15 03:21:36 +0200
committerGravatar Alexandre Alapetite <alexandre@alapetite.fr> 2015-05-15 03:21:36 +0200
commit256c8613a4046931dcd28ab22b6aebe8752a98c2 (patch)
tree2dbca039f47aec102a01e9a7d999fcfc26d294b4 /app
parentf4b0661f3db9f0bafb97636808ddc2274010d440 (diff)
First draft of PubSubHubbub
https://github.com/FreshRSS/FreshRSS/issues/312 Requires setting base_url in config.php. Currently using the filesystem (no change to the database)
Diffstat (limited to 'app')
-rwxr-xr-xapp/Controllers/feedController.php55
-rw-r--r--app/Models/Feed.php69
2 files changed, 103 insertions, 21 deletions
diff --git a/app/Models/Feed.php b/app/Models/Feed.php
index 85fb173ec..dcf083ea8 100644
--- a/app/Models/Feed.php
+++ b/app/Models/Feed.php
@@ -19,6 +19,8 @@ class FreshRSS_Feed extends Minz_Model {
private $ttl = -2;
private $hash = null;
private $lockPath = '';
+ private $hubUrl = '';
+ private $selfUrl = '';
public function __construct($url, $validate=true) {
if ($validate) {
@@ -226,6 +228,11 @@ class FreshRSS_Feed extends Minz_Model {
throw new FreshRSS_Feed_Exception(($errorMessage == '' ? 'Feed error' : $errorMessage) . ' [' . $url . ']');
}
+ $links = $feed->get_links('self');
+ $this->selfUrl = isset($links[0]) ? $links[0] : null;
+ $links = $feed->get_links('hub');
+ $this->hubUrl = isset($links[0]) ? $links[0] : null;
+
if ($loadDetails) {
// si on a utilisé l'auto-discover, notre url va avoir changé
$subscribe_url = $feed->subscribe_url(false);
@@ -259,7 +266,7 @@ class FreshRSS_Feed extends Minz_Model {
}
}
- private function loadEntries($feed) {
+ public function loadEntries($feed) {
$entries = array();
foreach ($feed->get_items() as $item) {
@@ -333,4 +340,64 @@ class FreshRSS_Feed extends Minz_Model {
function unlock() {
@unlink($this->lockPath);
}
+
+ //<PubSubHubbub>
+
+ function pubSubHubbubPrepare() {
+ $secret = '';
+ if (FreshRSS_Context::$system_conf->base_url && $this->hubUrl && $this->selfUrl) {
+ $path = PSHB_PATH . '/feeds/' . base64url_encode($this->selfUrl);
+ if (!file_exists($path . '/hub.txt')) {
+ @mkdir($path, 0777, true);
+ file_put_contents($path . '/hub.txt', $this->hubUrl);
+ $secret = sha1(FreshRSS_Context::$system_conf->salt . uniqid(mt_rand(), true));
+ file_put_contents($path . '/secret.txt', $secret);
+ @mkdir(PSHB_PATH . '/secrets/');
+ file_put_contents(PSHB_PATH . '/secrets/' . $secret . '.txt', base64url_encode($this->selfUrl));
+ Minz_Log::notice('PubSubHubbub prepared for ' . $this->url);
+ file_put_contents(USERS_PATH . '/_/log_pshb.txt', date('c') . "\t" .
+ 'PubSubHubbub prepared for ' . $this->url . "\n", FILE_APPEND);
+ }
+ $path .= '/' . base64url_encode($this->url);
+ $currentUser = Minz_Session::param('currentUser');
+ if (ctype_alnum($currentUser) && !file_exists($path . '/' . $currentUser . '.txt')) {
+ @mkdir($path, 0777, true);
+ touch($path . '/' . $currentUser . '.txt');
+ }
+ }
+ return $secret;
+ }
+
+ //Parameter true to subscribe, false to unsubscribe.
+ function pubSubHubbubSubscribe($state, $secret = '') {
+ if (FreshRSS_Context::$system_conf->base_url && $this->hubUrl && $this->selfUrl) {
+ $callbackUrl = checkUrl(FreshRSS_Context::$system_conf->base_url . 'api/pshb.php?s=' . $secret);
+ if ($callbackUrl == '') {
+ return false;
+ }
+
+ $ch = curl_init();
+ curl_setopt_array($ch, array(
+ CURLOPT_URL => $this->hubUrl,
+ CURLOPT_FOLLOWLOCATION => true,
+ CURLOPT_RETURNTRANSFER => true,
+ CURLOPT_USERAGENT => _t('gen.freshrss') . '/' . FRESHRSS_VERSION . ' (' . PHP_OS . '; ' . FRESHRSS_WEBSITE . ')',
+ CURLOPT_POSTFIELDS => 'hub.verify=sync'
+ . '&hub.mode=' . ($state ? 'subscribe' : 'unsubscribe')
+ . '&hub.topic=' . urlencode($this->selfUrl)
+ . '&hub.callback=' . urlencode($callbackUrl)
+ )
+ );
+ $response = curl_exec($ch);
+ $info = curl_getinfo($ch);
+
+ file_put_contents(USERS_PATH . '/_/log_pshb.txt', date('c') . "\t" .
+ 'PubSubHubbub ' . ($state ? 'subscribe' : 'unsubscribe') . ' to ' . $this->selfUrl .
+ ' with callback ' . $callbackUrl . ': ' . $info['http_code'] . ' ' . $response . "\n", FILE_APPEND);
+ return substr($info['http_code'], 0, 1) == '2';
+ }
+ return false;
+ }
+
+ //</PubSubHubbub>
}
diff --git a/app/Controllers/feedController.php b/app/Controllers/feedController.php
index 0443b4159..9117da639 100755
--- a/app/Controllers/feedController.php
+++ b/app/Controllers/feedController.php
@@ -168,6 +168,7 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
// Ok, feed has been added in database. Now we have to refresh entries.
$feed->_id($id);
$feed->faviconPrepare();
+ $feed->pubSubHubbubPrepare();
$is_read = FreshRSS_Context::$user_conf->mark_when['reception'] ? 1 : 0;
@@ -261,12 +262,13 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
* This action actualizes entries from one or several feeds.
*
* Parameters are:
- * - id (default: false)
+ * - id (default: false): Feed ID
+ * - url (default: false): Feed URL
* - force (default: false)
* If id is not specified, all the feeds are actualized. But if force is
* false, process stops at 10 feeds to avoid time execution problem.
*/
- public function actualizeAction() {
+ public function actualizeAction($simplePie = null) {
@set_time_limit(300);
$feedDAO = FreshRSS_Factory::createFeedDao();
@@ -274,14 +276,15 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
Minz_Session::_param('actualize_feeds', false);
$id = Minz_Request::param('id');
+ $url = Minz_Request::param('url');
$force = Minz_Request::param('force');
// Create a list of feeds to actualize.
// If id is set and valid, corresponding feed is added to the list but
// alone in order to automatize further process.
$feeds = array();
- if ($id) {
- $feed = $feedDAO->searchById($id);
+ if ($id || $url) {
+ $feed = $id ? $feedDAO->searchById($id) : $feedDAO->searchByUrl($url);
if ($feed) {
$feeds[] = $feed;
}
@@ -302,8 +305,11 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
}
try {
- // Load entries
- $feed->load(false);
+ if ($simplePie) {
+ $feed->loadEntries($simplePie); //Used by PubSubHubbub
+ } else {
+ $feed->load(false);
+ }
} catch (FreshRSS_Feed_Exception $e) {
Minz_Log::notice($e->getMessage());
$feedDAO->updateLastUpdate($feed->id(), true);
@@ -404,7 +410,16 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
$feedDAO->updateFeed($feed->id(), array('url' => $feed->url()));
}
- $feed->faviconPrepare();
+ if ($simplePie === null) {
+ $feed->faviconPrepare();
+ if ($feed->url() === 'http://push-pub.appspot.com/feed') {
+ $secret = $feed->pubSubHubbubPrepare();
+ if ($secret != '') {
+ Minz_Log::debug('PubSubHubbub subscribe ' . $feed->url());
+ $feed->pubSubHubbubSubscribe(true, $secret);
+ }
+ }
+ }
$feed->unlock();
$updated_feeds++;
unset($feed);
@@ -427,20 +442,20 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
Minz_Session::_param('notification', $notif);
// No layout in ajax request.
$this->view->_useLayout(false);
- return;
- }
-
- // Redirect to the main page with correct notification.
- if ($updated_feeds === 1) {
- $feed = reset($feeds);
- Minz_Request::good(_t('feedback.sub.feed.actualized', $feed->name()), array(
- 'params' => array('get' => 'f_' . $feed->id())
- ));
- } elseif ($updated_feeds > 1) {
- Minz_Request::good(_t('feedback.sub.feed.n_actualized', $updated_feeds), array());
} else {
- Minz_Request::good(_t('feedback.sub.feed.no_refresh'), array());
+ // Redirect to the main page with correct notification.
+ if ($updated_feeds === 1) {
+ $feed = reset($feeds);
+ Minz_Request::good(_t('feedback.sub.feed.actualized', $feed->name()), array(
+ 'params' => array('get' => 'f_' . $feed->id())
+ ));
+ } elseif ($updated_feeds > 1) {
+ Minz_Request::good(_t('feedback.sub.feed.n_actualized', $updated_feeds), array());
+ } else {
+ Minz_Request::good(_t('feedback.sub.feed.no_refresh'), array());
+ }
}
+ return $updated_feeds;
}
/**
@@ -261,12 +262,13 @@ class FreshRSS_feed_Controller extends Minz_ActionController {
* - force (default: false)
- * If id is not specified, all the feeds are actualized. But if force is
+ * If id and url are not specified, all the feeds are actualized. But if force is
* false, process stops at 10 feeds to avoid time execution problem.
*/